#!/bin/sh
# This script shows an example of running the sequence classifier on the subjectivity or sentiment datasets.
#
# $1 = dataset, either "subj" or "senti.binary"
# $2 = stepsize (I usually use 0.1)
# $3 = minibatch size (I use 10)
# $4 = num epochs (I use 5 or 10)
# $5 = softmax dropout probability (I use -1.0, i.e., no dropout, but you might want to try different values)
# $6 = input-hidden dropout prob (I use -1.0, again try different values here)
# $7 = hidden-hidden dropout prob (only used if multiple layers used) (I use -1.0, but I haven't tried many deep RNNs here)
# $8 = update word vectors? (true or false)
# $9 = word vectors to use, e.g., "paragram"
# $10 = algorithm (adadelta, sgd_momentum, or adagrad; I usually use adagrad)
# $11 = gradient norm threshold (<=0 to not use gradient scaling; I usually set this large, like 100.0, which basically means no threshold)
# $12 = configuration of forward LSTMs; run ./run_seqclassify with no arguments to see format for this string
# $13 = backward LSTMs
# $14 = forward GRUs
# $15 = backward GRUs
# $16 = convnet filter configurations; run ./run_seqclassify with no arguments to see format for this string
# $17 = suffix for log file -- specifies additional string to include in the log filename, as some of the above is not automatically included there, like stepsize, dropout rates, etc. See below for format of log filename.
# $18 = how often to decode test data. If 0, will decode every 1/5 of the way through the training data. Use 3000 for senti.binary and 2000 for subj.
./run_seqclassify \
train_file=data/$1/$1.train \
dev_file=data/$1/$1.dev \
test_file=data/$1/$1.test \
stepsize=$2 \
minibatch_size=$3 \
num_epochs=$4 \
softmax_dropout_rate=$5 \
input_dropout_rate=$6 \
hidden_dropout_rate=$7 \
update_word_vectors=$8 \
word_vector_file=data/wordvects.$9 \
algorithm=${10} \
gradient_norm_threshold=${11} \
forward_lstms=${12} \
backward_lstms=${13} \
forward_grus=${14} \
backward_grus=${15} \
cnn_filters=${16} \
decode_every_k=${18} \
2> log.seqclassify.$1.$9.uwv-$8.${12}-${13}-${14}-${15}.cnn${16}.${17}.deck${18}.stderr

