#include "formula.h"
#include "formula_pool.h"

void FormulaPool::initialize(unsigned int capacity) {
	m_current = 0;
	m_formulas.clear();
	m_formulaPointers.clear();
	m_formulaPointers.reserve(capacity);
	for (unsigned int i = 0; i < capacity; ++i) {
		createNewFormula();
	}
}

// Find an unused Formula and return a pointer to it.
Formula* FormulaPool::getFormula() {
	// If we are at the end of the Formula pointer list, create a new Formula.
	if (m_current >= m_formulaPointers.size()) {
		if (m_current > m_formulaPointers.size()) {
			cerr << "Error: in FormulaPool::getFormula(), m_current (" << m_current
				 << ") > m_formulaPointers.size() (" << m_formulaPointers.size() << ")" << endl;
			return NULL;
		} else {
			createNewFormula();
		}
	}
	// The Formula pointer we will return is the one at the end of m_formulaPointers.
	Formula* ret = m_formulaPointers[m_current];
	// Increment m_current so that we will be looking to return the next Formula the next time around.
	++m_current;
	return ret;
}

void FormulaPool::freeAllFormulas() {
	m_current = 0;
}

void FormulaPool::resetAllFormulaComputedFlags() {
	//cerr << "Resetting " << m_current << " formulas" << endl;
	for (unsigned int i = 0; i < m_current; ++i) {
		m_formulaPointers[i]->changedParamValues(false);
	}
}

int FormulaPool::getNumberOfFormulasUsed() const {
	return m_current;
}

void FormulaPool::createNewFormula() {
	Formula f;
	// If using C++11, should use emplace_back instead of push_back below.
	m_formulas.push_back(f);
	m_formulaPointers.push_back(&(m_formulas.back()));
}
