#ifndef CONVOLUTIONAL_NEURAL_NETWORK_H_
#define CONVOLUTIONAL_NEURAL_NETWORK_H_

#include <fstream>
#include <iostream>
#include <vector>
#include <string>
#include <set>
#include <queue>
#include <list>
#include "../../fdag/model.h"
#include "../classifier_utils.h"

//using namespace std;

struct Filter {
	// The width of the filter is the number of words it considers in the sentence.
	int m_width;
	// If original word vectors are k-dimensional, weight vector will have m_width * k entries.
	vector<PARAM_ID> m_weights;
	// The bias weight is a scalar.
	PARAM_ID m_bias;
};

class ConvolutionalNeuralNetwork {
public:
	void initializeParameters(Model& model, int dataInputDimension, int numLabels, const string& filters,
		bool initializeSoftmaxParameters);
	int getNumberOfFilters() const;
	Formula* createFilterPoolingOutputNode(Model& model, SequenceDatum& datum, unsigned int i);

	//Formula* createHiddenNodeAtTimeAtDepth(Model& model, SequenceDatum& datum, unsigned int sequenceIndex, unsigned int i,
	//	int t, unsigned int depth, double inputDropoutRate, double hiddenDropoutRate, int terminationTime = 0);
	//void getGateReport(Model& model, SequenceDatum& datum, unsigned int sequenceIndex, int t, unsigned int depth,
	//	double& inputGateSum, double& forgetGateSum, double& outputGateSum, int terminationTime = 0);
	//void resetForNewExample(unsigned int sequenceIndex, int exampleLength);
	void resetForNewExample();

	vector<vector<PARAM_ID> > m_softmaxWeights;
	vector<PARAM_ID> m_softmaxBias;

private:
	//void initializeAllParameters(const vector<WordSequenceDatum>& trainData, const vector<WordSequenceDatum>& devData,
	//	const vector<WordSequenceDatum>& testData, bool initWordVectorsRandomly, const string& filters);
	// Precomputes the Formula's needed for the words in the datum, where v holds raw word vectors
	// for the words and vp holds the associated parameter IDs (only used if word vectors are being updated).
	// The result is that vf holds pointers to either constant Formula nodes or parameter Formula nodes, depending
	// on whether word vectors are being updated.
	//void precomputeGivenWordVectorFormulas(const vector<const vector<double>* >& v,
	//	const vector<const vector<PARAM_ID>* >& vp, unsigned int dim, bool updateWordVectors, unsigned int numWords,
	//	vector< vector< Formula* > >& vf);
	//void precomputeGivenWordVectorFormulas(const vector<const vector<double>* >& v, const vector<const vector<PARAM_ID>* >& vp,
	//	unsigned int dim, unsigned int numWords, vector< vector< Formula* > >& vf);
	//void precomputeWordVectorFormulas(WordSequenceDatum& datum, unsigned int inputDimension, bool updateWordVectors);
	Formula* createFilterOutputNodeAtPosition(Model& model, Filter& filter, const SequenceDatum& datum, int position);

	void printFilterNearestNeighbors(vector<WordSequenceDatum>& trainData);
	void printNearestNeighbors(vector<WordSequenceDatum>& trainData, vector<WordSequenceDatum>& devData);

	// The input dimension is the size of the given word vectors.
	int m_inputDimension;
	// The number of class labels.
	unsigned int m_numLabels;
	// The number of filters in the CNN:
	unsigned int m_numFilters;

	//vector<PARAM_ID> m_nbowBias;

	vector<Filter> m_filters;

	vector<Formula*> m_ciNodes;	// Pointers to cached Formulas for c_i nodes, which are NULL if unset (or dropped out).

	//bool m_updateWordVectors;
	//bool m_useStochasticPooling;
	//bool m_useCosineSimilarityFilters;
	//int m_maxSequenceLength;
};

#endif
