#ifndef UTILS_H_
#define	UTILS_H_

#include <vector>
#include <string>

using namespace std;

extern const double neg_inf;
extern const double pos_inf;
extern const float neg_inf_float;
extern const float pos_inf_float;

void tokenize(const string& str, char delimiter, vector<string>& tokens);

double logAdd(const double lx, const double ly);

double logSubtract(const double lx, const double ly);

string itos(int i);

string ftos(double d);

int roundToNearestInt(double x);

double computeTimeDifference(struct timeval& time1, struct timeval& time2);

// Returns a random integer between 0 and max, inclusive.
int getRandomInteger(int max);

// Returns a random double between 0 and 1.
double getRandomDouble();

void readLinesFromFile(const string& inputFile, vector<string>& lines);

string lowercase(const string& s);

string trimString(const string& str);

string changeDigitsToGivenChar(const string& s, char givenChar, bool* foundDigits);

double computeVectorNorm(const vector<double>& v);

double standardDeviation(const vector<double>& v);

// Given two data arrays, data1[0..n-1] and data2[0..n-1], this routine returns their
// sum squared difference of ranks as d and the Spearman's rank correlation as rs.
// The external routines convertSortedVectorToRanks and sortTwoVectorsByFirstVector are used.
// This implementation was adapted from http://enviroinsite.blogspot.com/2014/01/spearman-rank-correlation-trend-test.html .
// It was checked that tie-handling matched the output of MatLab's corr(x, 'type', 'Spearman') function.
// Test cases:
//mm =
//0    6.0000
//2.0000   10.0000
//2.0000   10.0000
//1.5000   10.0000
//1.5000    6.0000
//2.0000   10.0000
//2.0000    2.0000
//rho = 0.3
//
//mm =
//         0    6.0000
//    2.0000   10.0000
//    2.0000   10.0000
//    1.5000   10.0100
//    1.5000    6.0000
//    2.0000   10.0000
//   -1.0000    2.0000
//rho = 0.676471
void spearmanCorrelation(const vector<double>& data1, const vector<double>& data2, double& d, double& rs);

void convertSortedVectorToRanks(vector<double>& w, double& s);

struct CompareDoublePairVectorFirstDouble {
	bool operator() (const pair<double, double>& a, const pair<double, double>& b) const {
		return a.first < b.first;
	}
};

void sortTwoVectorsByFirstVector(vector<double>& arr, vector<double>& brr);

#endif	/* UTILS_H_ */

